@@ -160,6 +160,11 @@ arma::mat matrix_coordinates(arma::mat X, arma::mat B){
     return(log(X) * B);
 }
 
+// [[Rcpp::export]]
+arma::mat sparse_coordinates(arma::mat X, arma::sp_mat B){
+  return(log(X) * B);
+}
+
 // [[Rcpp::export]]
 arma::mat coordinates_basis(arma::mat X, arma::mat B, bool sparse = false){
   if(sparse){
